package jdk.internal.org.objectweb.asm.util;

import jdk.internal.org.objectweb.asm.Type;
import jdk.internal.org.objectweb.asm.AnnotationVisitor;

public class CheckAnnotationAdapter extends AnnotationVisitor
{
    private final boolean named;
    private boolean end;
    
    public CheckAnnotationAdapter(AnnotationVisitor annotationVisitor) {
        this(annotationVisitor, true);
    }
    
    CheckAnnotationAdapter(AnnotationVisitor annotationVisitor, boolean named) {
        super(327680, annotationVisitor);
        this.named = named;
    }
    
    @Override
    public void visit(String name, Object value) {
        this.checkEnd();
        this.checkName(name);
        if (!(value instanceof Byte) && !(value instanceof Boolean) && !(value instanceof Character) && !(value instanceof Short) && !(value instanceof Integer) && !(value instanceof Long) && !(value instanceof Float) && !(value instanceof Double) && !(value instanceof String) && !(value instanceof Type) && !(value instanceof byte[]) && !(value instanceof boolean[]) && !(value instanceof char[]) && !(value instanceof short[]) && !(value instanceof int[]) && !(value instanceof long[]) && !(value instanceof float[]) && !(value instanceof double[])) {
            throw new IllegalArgumentException("Invalid annotation value");
        }
        if (value instanceof Type && ((Type)value).getSort() == 11) {
            throw new IllegalArgumentException("Invalid annotation value");
        }
        if (this.av != null) {
            this.av.visit(name, value);
        }
        return;
    }
    
    @Override
    public void visitEnum(String name, String descriptor, String value) {
        this.checkEnd();
        this.checkName(name);
        CheckMethodAdapter.checkDesc(descriptor, false);
        if (value == null) {
            throw new IllegalArgumentException("Invalid enum value");
        }
        if (this.av != null) {
            this.av.visitEnum(name, descriptor, value);
        }
        return;
    }
    
    @Override
    public AnnotationVisitor visitAnnotation(String name, String descriptor) {
        this.checkEnd();
        this.checkName(name);
        CheckMethodAdapter.checkDesc(descriptor, false);
        return new CheckAnnotationAdapter((this.av == null) ? null : this.av.visitAnnotation(name, descriptor));
    }
    
    @Override
    public AnnotationVisitor visitArray(String name) {
        this.checkEnd();
        this.checkName(name);
        return new CheckAnnotationAdapter((this.av == null) ? null : this.av.visitArray(name), false);
    }
    
    @Override
    public void visitEnd() {
        this.checkEnd();
        this.end = true;
        if (this.av != null) {
            this.av.visitEnd();
        }
    }
    
    private void checkEnd() {
        if (this.end) {
            throw new IllegalStateException("Cannot call a visit method after visitEnd has been called");
        }
        return;
    }
    
    private void checkName(String s) {
        if (this.named && s == null) {
            throw new IllegalArgumentException("Annotation value name must not be null");
        }
        return;
    }
}
